###########################
# PlonePlaceMap Installer #
###########################

from AccessControl.Permission import registerPermissions
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import ToolInit
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.CMFPlone.migrations.migration_util import safeEditProperty
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.public import listTypes
from Products.PlonePlaceMap import globalli

from StringIO import StringIO

configlets = \
( { 'id'         : 'PlonePlaceMap'
  , 'name'       : 'PlonePlaceMap'
  , 'action'     : 'string:${portal_url}/conf'
  , 'category'   : 'Products'
  , 'appId'      : 'PlonePlaceMap'
  , 'permission' : ManagePortal
  , 'imageUrl'  : 'book_icon.gif'
  }
,
)

PROPERTY_SHEET = 'PlonePlaceMap_properties'
PROPERTY_FIELD = 'keyslist'
# This is a  generic set of 5 host&key for google maps
KEYS = ["http://127.0.0.1:8080|ABQIAAAADBVqgJsmtwM-lc43tUxzuBTBfUk9TZrBRaIteybtnU2KziHEpRRam3dv-nKMkF8uTwWueguM1l9MVg","2key","3key","4key","5key" ]
	
def install(self):
    out = StringIO()
    #Add 2 new view into folder and topic
    addView(self, 'Folder', 'folder_maps', out)
    addView(self, 'Topic', 'topic_maps', out)
    # Install the subskin
    install_subskin(self,out,'PlonePlaceMap',globalli)
    # Adding a configlets
    addConfiglets(self, out)
    # Adding a property
    addProperty(self, out)
    return out.getvalue()

def uninstall(self):
    out = StringIO()
    # Removing the 2 views
    removeView(self, 'Folder', 'folder_maps', out)
    removeView(self, 'Topic', 'topic_maps', out)
    # Remove the configlets and the property
    removeConfiglets(self, out)
    removeProperty(self, out)
    return out.getvalue()

def addProperty(self, out):
    portal_props = getToolByName(self, 'portal_properties')
    # If PlonePlaceMap_properties isn't present in properties adding it
    if not hasattr(portal_props, PROPERTY_SHEET):
        portal_props.addPropertySheet(PROPERTY_SHEET, 'PlonePlaceMap Properties')
        out.write('Property %s was added\n' % PROPERTY_SHEET)
    ap = getattr(portal_props, PROPERTY_SHEET)
    # If the height field isn't present in PlonePlaceMap_properties adding it
    if not hasattr(ap,"height"):
        safeEditProperty(ap,"height",500,'int')
        out.write("Field height was added in %s\n" %PROPERTY_SHEET)
    # If the color field isn't present in PlonePlaceMap_properties adding it
    if not hasattr(ap,"color"):
        safeEditProperty(ap,"color","red",'string')
        out.write("Field color was added in %s\n" %PROPERTY_SHEET)
    # If the parameters field isn't present in PlonePlaceMap_properties adding it
    if not hasattr(ap,"parameters"):
        safeEditProperty(ap,"parameters","1111",'string')
        out.write("Field parameters was added in %s\n" %PROPERTY_SHEET)
    # If the keyslist field isn't present in PlonePlaceMap_properties adding it
    if not hasattr(ap, PROPERTY_FIELD):
        safeEditProperty(ap, PROPERTY_FIELD, KEYS, 'lines')
        out.write("Field keyslist was added in %s\n" %PROPERTY_SHEET)
	
def removeProperty(self, out):
    portal_props = getToolByName(self, 'portal_properties')
    if hasattr(portal_props, PROPERTY_SHEET):
        portal_props.manage_delObjects(PROPERTY_SHEET)
        out.write('%s was remove from properties\n' % PROPERTY_SHEET)

def addConfiglets(self, out):
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in configlets:
            out.write('Confiflet %s is added\n' % conf['id'])
            configTool.registerConfiglet(**conf)

def removeConfiglets(self, out):
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in configlets:
            out.write('Configlet %s is removed\n' % conf['id'])
            configTool.unregisterConfiglet(conf['id'])

def install_subskin(self,out,skin_name,globals):
    skinstool=getToolByName(self,'portal_skins')
    if skin_name not in skinstool.objectIds():
        addDirectoryViews(skinstool,'skins',globals)

    for skinName in skinstool.getSkinSelections():
        path=skinstool.getSkinPath(skinName)
	path=[i.strip() for i in path.split(',')]
	try:
	    if skin_name not in path:
	        path.insert(path.index('custom')+1,skin_name)
        except ValueError:
	    if skin_name not in path:
	        path.append(skin_name)

	path=",".join(path)
	skinstool.addSkinSelection(skinName,path)

def addView(self, portal_type,vista, out):
    types_tool = getToolByName(self, 'portal_types')
    if types_tool is not None:
        fti = getattr(types_tool, portal_type, None)
        if fti is not None:
            try:
                views = fti.view_methods + (vista,)
                fti.manage_changeProperties(view_methods=views)
                print >> out, "The view %s is added" % vista
            except:
                print >> out, "The view %s is not added,failure" % vista

def removeView(self, portal_type, vista, out):
    types_tool = getToolByName(self, 'portal_types')
    if types_tool is not None:
        fti = getattr(types_tool, portal_type, None)
        if fti is not None:
            try:
                source = fti.view_methods
                views = []
                for view in source:
                    if view not in vista:
                        views.append(view)
                fti.manage_changeProperties(view_methods=tuple(views))
                print >> out, "The view %s is removed" % vista
            except:
                print >> out, "The view %s is not removed,failure" % vista
